@@ -61,6 +61,17 @@ pub trait Float:
     /// Truncates the fractional part from this value.
     fn trunc(self) -> Self;
 
+    fn sin(self) -> Self;
+    fn cos(self) -> Self;
+    fn tan(self) -> Self;
+
+    fn asin(self) -> Self;
+    fn acos(self) -> Self;
+    fn atan(self) -> Self;
+    fn atan2(self, _: Self) -> Self;
+
+    fn total_eq(self, _: Self) -> bool;
+
     /// Type that implements [`std::cmp::Ord`], which this floating point type can be trivially converted to.
     type Ord: Ord;
     /// Converts this floating point number to a type that implements total ordering. Conversion should be trivial.  
@@ -109,6 +120,55 @@ macro_rules! impl_float {
                 <$f>::trunc(self)
             }
 
+            fn sin(self) -> Self {
+                <$f>::sin(self)
+            }
+            #[inline]
+            fn cos(self) -> Self {
+                <$f>::cos(self)
+            }
+            #[inline]
+            fn tan(self) -> Self {
+                <$f>::tan(self)
+            }
+
+            #[inline]
+            fn asin(self) -> Self {
+                <$f>::asin(self)
+            }
+            #[inline]
+            fn acos(self) -> Self {
+                <$f>::acos(self)
+            }
+            #[inline]
+            fn atan(self) -> Self {
+                <$f>::atan(self)
+            }
+            #[inline]
+            fn atan2(self, b: Self) -> Self {
+                <$f>::atan2(self, b)
+            }
+
+            #[inline]
+            fn total_eq(self, rhs: Self) -> bool {
+                let a = self.to_bits();
+                let b = rhs.to_bits();
+
+                // disregard the sign bit when comparing zeros.
+                if a << 1 == 0 {
+                    return zero(b);
+
+                    #[cold]
+                    fn zero(b: $i) -> bool {
+                        b << 1 == 0
+                    }
+                }
+                // compare any other numbers directly.
+                else {
+                    a == b
+                }
+            }
+
             type Ord = $i;
             #[inline]
             fn to_ord(self) -> Self::Ord {
@@ -147,6 +207,8 @@ impl_float!(f64, u64);
 
 /// A unit of measurement for an [`Angle`].
 pub trait Unit<F: Float>: Sized {
+    const DBG_NAME: &'static str;
+
     /// Number for a quarter turn around a circle in this unit space.
     const QUARTER_TURN: F;
     /// Number for a half turn around a circle in this unit space.
@@ -163,27 +225,6 @@ pub trait Unit<F: Float>: Sized {
 #[repr(transparent)]
 pub struct Angle<F: Float, U: Unit<F>>(F, PhantomData<U>);
 
-impl<F: Float, U: Unit<F>> Default for Angle<F, U> {
-    #[inline]
-    fn default() -> Self {
-        Self(F::ZERO, PhantomData)
-    }
-}
-impl<F: Float, U: Unit<F>> Clone for Angle<F, U> {
-    #[inline]
-    fn clone(&self) -> Self {
-        Self(self.0, PhantomData)
-    }
-}
-impl<F: Float, U: Unit<F>> Copy for Angle<F, U> {}
-
-impl<F: Float, U: Unit<F>> PartialEq for Angle<F, U> {
-    #[inline]
-    fn eq(&self, rhs: &Self) -> bool {
-        self.0.to_ord() == rhs.0.to_ord()
-    }
-}
-impl<F: Float, U: Unit<F>> Eq for Angle<F, U> {}
 impl<F: Float, U: Unit<F>> PartialOrd for Angle<F, U> {
     #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
@@ -200,13 +241,7 @@ impl<F: Float, U: Unit<F>> Ord for Angle<F, U> {
 impl<F: Float + fmt::Debug, U: Unit<F>> fmt::Debug for Angle<F, U> {
     #[inline]
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        fmt::Debug::fmt(&self.0, f)
-    }
-}
-impl<F: Float + fmt::Display, U: Unit<F>> fmt::Display for Angle<F, U> {
-    #[inline]
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        U::display(self.0, f)
+        f.debug_tuple(U::DBG_NAME).field(&self.0).finish()
     }
 }
 
@@ -250,73 +285,238 @@ impl<F: Float, U: Unit<F>> Angle<F, U> {
     }
 }
 
-impl<F: Float, U: Unit<F>> Add for Angle<F, U> {
-    type Output = Self;
+/// An angle that wraps between a negative half turn and a positive half turn.
+#[repr(transparent)]
+pub struct Wrap<F: Float, U: Unit<F>>(Angle<F, U>);
+
+impl<F: Float + fmt::Debug, U: Unit<F>> fmt::Debug for Wrap<F, U> {
     #[inline]
-    fn add(self, rhs: Self) -> Self {
-        Self::new(self.0 + rhs.0)
-    }
-}
-impl<F: Float, U: Unit<F>> AddAssign for Angle<F, U> {
-    #[inline]
-    fn add_assign(&mut self, rhs: Self) {
-        self.0 += rhs.0;
-        debug_assert!(self.0.is_finite());
-    }
-}
-impl<F: Float, U: Unit<F>> Sub for Angle<F, U> {
-    type Output = Self;
-    #[inline]
-    fn sub(self, rhs: Self) -> Self {
-        Self::new(self.0 - rhs.0)
-    }
-}
-impl<F: Float, U: Unit<F>> SubAssign for Angle<F, U> {
-    #[inline]
-    fn sub_assign(&mut self, rhs: Self) {
-        self.0 -= rhs.0;
-        debug_assert!(self.0.is_finite());
-    }
-}
-impl<F: Float, U: Unit<F>> Neg for Angle<F, U> {
-    type Output = Self;
-    #[inline]
-    fn neg(self) -> Self {
-        Self(-self.0, PhantomData)
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_tuple("Wrap").field(&self.0).finish()
     }
 }
 
-impl<F: Float, U: Unit<F>> Mul<F> for Angle<F, U> {
-    type Output = Self;
+impl<F: Float, U: Unit<F>> Wrap<F, U> {
+    /// Zero angle, additive identity.
+    pub const ZERO: Self = Self(Angle::ZERO);
+    /// Half turn around a circle. Equal to π/2 radians or 90°.
+    pub const QUARTER_TURN: Self = Self(Angle::QUARTER_TURN);
+    /// Half turn around a circle. Equal to π radians or 180°.
+    pub const HALF_TURN: Self = Self(Angle::HALF_TURN);
+    /// Full turn around a circle. Equal to 2π radians or 360°;
+    pub const FULL_TURN: Angle<F, U> = Angle::FULL_TURN;
+
+    /// Creates a new angle, wrapping between a negative half turn and a positive half turn.
+    pub fn wrap(val: F) -> Self {
+        let val = (-val + U::HALF_TURN).rem_euclid(U::FULL_TURN) - U::HALF_TURN;
+        Self(Angle::new(-val))
+    }
+    /// Creates a new angle, without checking if it's in range.
     #[inline]
-    fn mul(self, rhs: F) -> Self {
-        Self::new(self.0 * rhs)
+    fn new_unchecked(val: F) -> Self {
+        Self(Angle::new(val))
+    }
+
+    /// Gets the value of this angle.
+    #[inline]
+    pub fn val_raw(self) -> F {
+        self.0 .0
+    }
+    /// Gets the inner representation of this angle.
+    #[inline]
+    pub fn inner(self) -> Angle<F, U> {
+        self.0
+    }
+    /// Returns the magnitude (absolute value) of this angle.
+    #[inline]
+    pub fn mag(self) -> Angle<F, U> {
+        self.0.mag()
     }
 }
-impl<F: Float, U: Unit<F>> MulAssign<F> for Angle<F, U> {
+
+impl<F: Float, U: Unit<F>> From<Wrap<F, U>> for Angle<F, U> {
     #[inline]
-    fn mul_assign(&mut self, rhs: F) {
-        self.0 *= rhs;
-        debug_assert!(self.0.is_finite());
+    fn from(val: Wrap<F, U>) -> Self {
+        val.0
     }
 }
-impl<F: Float, U: Unit<F>> Div<F> for Angle<F, U> {
-    type Output = Self;
-    #[inline]
-    fn div(self, rhs: F) -> Self {
-        Self::new(self.0 / rhs)
+
+macro_rules! impl_traits {
+    ($ang: ident : $new: ident) => {
+        impl<F: Float, U: Unit<F>> Default for $ang<F, U> {
+            #[inline]
+            fn default() -> Self {
+                Self::$new(F::ZERO)
+            }
+        }
+
+        impl<F: Float, U: Unit<F>> Clone for $ang<F, U> {
+            #[inline]
+            fn clone(&self) -> Self {
+                Self::$new(self.val_raw())
+            }
+        }
+        impl<F: Float, U: Unit<F>> Copy for $ang<F, U> {}
+
+        impl<F: Float, U: Unit<F>> PartialEq for $ang<F, U> {
+            #[inline]
+            fn eq(&self, rhs: &Self) -> bool {
+                self.val_raw().total_eq(rhs.val_raw())
+            }
+        }
+        impl<F: Float, U: Unit<F>> Eq for $ang<F, U> {}
+
+        impl<F: Float + fmt::Display, U: Unit<F>> fmt::Display for $ang<F, U> {
+            fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+                U::display(self.val_raw(), f)
+            }
+        }
+    };
+    ($($ang: ident : $new: ident),*) => {
+        $(impl_traits!($ang : $new);)*
     }
 }
-impl<F: Float, U: Unit<F>> DivAssign<F> for Angle<F, U> {
-    #[inline]
-    fn div_assign(&mut self, rhs: F) {
-        self.0 /= rhs;
-        debug_assert!(self.0.is_finite());
+
+impl_traits!(Angle: new, Wrap: new_unchecked);
+
+macro_rules! impl_ops {
+    ($ang: ident : $new: ident) => {
+        impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> Add<Rhs> for $ang<F, U> {
+            type Output = Self;
+            #[inline]
+            fn add(self, rhs: Rhs) -> Self {
+                Self::$new(self.val_raw() + rhs.into().val_raw())
+            }
+        }
+        impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> AddAssign<Rhs> for $ang<F, U> {
+            #[inline]
+            fn add_assign(&mut self, rhs: Rhs) {
+                *self = *self + rhs;
+            }
+        }
+
+        impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> Sub<Rhs> for $ang<F, U> {
+            type Output = Self;
+            #[inline]
+            fn sub(self, rhs: Rhs) -> Self {
+                Self::$new(self.val_raw() - rhs.into().val_raw())
+            }
+        }
+        impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> SubAssign<Rhs> for $ang<F, U> {
+            #[inline]
+            fn sub_assign(&mut self, rhs: Rhs) {
+                *self = *self - rhs;
+            }
+        }
+
+        impl<F: Float, U: Unit<F>> Neg for $ang<F, U> {
+            type Output = Self;
+            #[inline]
+            fn neg(self) -> Self {
+                Self::$new(-self.val_raw())
+            }
+        }
+
+        impl<F: Float, U: Unit<F>> Mul<F> for $ang<F, U> {
+            type Output = Self;
+            #[inline]
+            fn mul(self, rhs: F) -> Self {
+                Self::$new(self.val_raw() * rhs)
+            }
+        }
+        impl<F: Float, U: Unit<F>> MulAssign<F> for $ang<F, U> {
+            #[inline]
+            fn mul_assign(&mut self, rhs: F) {
+                *self = *self * rhs;
+            }
+        }
+
+        impl<F: Float, U: Unit<F>> Div<F> for $ang<F, U> {
+            type Output = Self;
+            #[inline]
+            fn div(self, rhs: F) -> Self {
+                Self::$new(self.val_raw() / rhs)
+            }
+        }
+        impl<F: Float, U: Unit<F>> DivAssign<F> for $ang<F, U> {
+            #[inline]
+            fn div_assign(&mut self, rhs: F) {
+                *self = *self / rhs;
+            }
+        }
+    };
+    ($($ang: ident : $new: ident),*) => {
+        $(impl_ops!($ang : $new);)*
     }
 }
 
+impl_ops!(Angle: new, Wrap: wrap);
+
+macro_rules! impl_trig {
+    ($ang: ident : $new: ident) => {
+        impl<F: Float, U: Unit<F>> $ang<F, U> {
+            /// Computes the sine of this angle.
+            #[inline]
+            pub fn sin(self) -> F {
+                let rad = self.val_raw() * (F::PI / U::HALF_TURN);
+                rad.sin()
+            }
+            /// Computes the cosine of this angle.
+            #[inline]
+            pub fn cos(self) -> F {
+                let rad = self.val_raw() * (F::PI / U::HALF_TURN);
+                rad.cos()
+            }
+            /// Computes both the sine and cosine of this angle.
+            #[inline]
+            pub fn sin_cos(self) -> (F, F) {
+                let rad = self.val_raw() * (F::PI / U::HALF_TURN);
+                (rad.sin(), rad.cos())
+            }
+            /// Computes the tangent of this angle.
+            #[inline]
+            pub fn tan(self) -> F {
+                let rad = self.val_raw() * (F::PI / U::HALF_TURN);
+                rad.tan()
+            }
+
+            /// Computes the arc-sine of the specified value.
+            #[inline]
+            pub fn asin(y: F) -> Self {
+                let rad = y.asin();
+                Self::$new(rad * (U::HALF_TURN / F::PI))
+            }
+            /// Computes the arc-cosine of the specified value.
+            #[inline]
+            pub fn acos(x: F) -> Self {
+                let rad = x.acos();
+                Self::$new(rad * (U::HALF_TURN / F::PI))
+            }
+            /// Computes the arc-tangent of the specified value.
+            #[inline]
+            pub fn atan(tan: F) -> Self {
+                let rad = tan.atan();
+                Self::$new(rad * (U::HALF_TURN / F::PI))
+            }
+            /// Computes the four-quadrant arc-tangent of the specified fraction.
+            #[inline]
+            pub fn atan2(y: F, x: F) -> Self {
+                let rad = y.atan2(x);
+                Self::$new(rad * (U::HALF_TURN / F::PI))
+            }
+        }
+    };
+    ($($ang: ident : $new: ident),*) => {
+        $(impl_trig!($ang : $new);)*
+    };
+}
+
+impl_trig!(Angle: new, Wrap: new_unchecked);
+
 pub struct Radians;
 impl<F: Float> Unit<F> for Radians {
+    const DBG_NAME: &'static str = "Rad";
+
     const QUARTER_TURN: F = F::PI_OVER_TWO;
     const HALF_TURN: F = F::PI;
     const FULL_TURN: F = F::TAU;
@@ -331,24 +531,23 @@ impl<F: Float> Unit<F> for Radians {
             write!(f, "+")?;
         }
         let val = val.abs();
-        if val == F::ZERO {
-            if let Some(prec) = f.precision() {
-                write!(f, "0.{:0<prec$}", 0)
-            } else {
-                write!(f, "0")
-            }
+        if let Some(prec) = f.precision() {
+            write!(f, "{:.prec$}", val / F::PI)?;
         } else {
-            if let Some(prec) = f.precision() {
-                write!(f, "{:.prec$}π", val / F::PI)
-            } else {
-                write!(f, "{}π", val / F::PI)
-            }
+            write!(f, "{}", val / F::PI)?;
         }
+        if val != F::ZERO {
+            write!(f, "π")?;
+        }
+        Ok(())
     }
 }
 
 /// An angle measured in radians.
 ///
+/// This type is guaranteed to be finite (in debug mode). As such, it implements total equality and ordering.
+/// Ordering for non-finite values is unspecified.
+///
 /// When formatting with [`Display`](fmt::Display), it will be shown as a multiple of π.
 pub type Rad<F> = Angle<F, Radians>;
 
@@ -363,9 +562,33 @@ impl<F: Float> Rad<F> {
         Deg::new(self.0 * (F::ONE_EIGHTY / F::PI))
     }
 }
+impl<F: Float> Wrap<F, Radians> {
+    /// Gets the value of this angle in radians.
+    #[inline]
+    pub fn val(self) -> F {
+        self.0 .0
+    }
+}
+
+/// A 32-bit angle measured in radians.
+///
+/// This type is guaranteed to be finite (in debug mode). As such, it implements total equality and ordering.
+/// Ordering for non-finite values is unspecified.
+///
+/// When formatting with [`Display`](fmt::Display), it will be shown as a multiple of π.
+pub type Rad32 = Rad<f32>;
+/// A 64-bit angle measured in radians.
+///
+/// This type is guaranteed to be finite (in debug mode). As such, it implements total equality and ordering.
+/// Ordering for non-finite values is unspecified.
+///
+/// When formatting with [`Display`](fmt::Display), it will be shown as a multiple of π.
+pub type Rad64 = Rad<f64>;
 
 pub struct Degrees;
 impl<F: Float> Unit<F> for Degrees {
+    const DBG_NAME: &'static str = "Deg";
+
     const QUARTER_TURN: F = F::NINETY;
     const HALF_TURN: F = F::ONE_EIGHTY;
     const FULL_TURN: F = F::THREE_SIXTY;
@@ -407,76 +630,11 @@ impl<F: Float> Deg<F> {
     pub fn val(self) -> F {
         self.0
     }
+    /// Converts this angle to radians.
     pub fn rad(self) -> Rad<F> {
         Rad::new(self.0 * (F::PI / F::ONE_EIGHTY))
     }
 }
-
-/// An angle that wraps between a negative half turn and a positive half turn.
-#[repr(transparent)]
-pub struct Wrap<F: Float, U: Unit<F>>(Angle<F, U>);
-
-impl<F: Float, U: Unit<F>> Default for Wrap<F, U> {
-    #[inline]
-    fn default() -> Self {
-        Self(Default::default())
-    }
-}
-impl<F: Float, U: Unit<F>> Clone for Wrap<F, U> {
-    #[inline]
-    fn clone(&self) -> Self {
-        Self(self.0)
-    }
-}
-impl<F: Float, U: Unit<F>> Copy for Wrap<F, U> {}
-impl<F: Float, U: Unit<F>> PartialEq for Wrap<F, U> {
-    #[inline]
-    fn eq(&self, rhs: &Self) -> bool {
-        self.0 == rhs.0
-    }
-}
-impl<F: Float, U: Unit<F>> Eq for Wrap<F, U> {}
-
-impl<F: Float, U: Unit<F>> Wrap<F, U> {
-    /// Zero angle, additive identity.
-    pub const ZERO: Self = Self(Angle::ZERO);
-    /// Half turn around a circle. Equal to π/2 radians or 90°.
-    pub const QUARTER_TURN: Self = Self(Angle::QUARTER_TURN);
-    /// Half turn around a circle. Equal to π radians or 180°.
-    pub const HALF_TURN: Self = Self(Angle::HALF_TURN);
-    /// Full turn around a circle. Equal to 2π radians or 360°;
-    pub const FULL_TURN: Angle<F, U> = Angle::FULL_TURN;
-
-    /// Creates a new angle, wrapping between a negative half turn and a positive half turn.
-    pub fn wrap(val: F) -> Self {
-        let val = (-val + U::HALF_TURN).rem_euclid(U::FULL_TURN) - U::HALF_TURN;
-        Self(Angle::new(-val))
-    }
-
-    /// Gets the value of this angle.
-    #[inline]
-    pub fn val_raw(self) -> F {
-        self.0 .0
-    }
-    /// Gets the inner representation of this angle.
-    #[inline]
-    pub fn inner(self) -> Angle<F, U> {
-        self.0
-    }
-    /// Returns the magnitude (absolute value) of this angle.
-    #[inline]
-    pub fn mag(self) -> Angle<F, U> {
-        self.0.mag()
-    }
-}
-
-impl<F: Float> Wrap<F, Radians> {
-    /// Gets the value of this angle in radians.
-    #[inline]
-    pub fn val(self) -> F {
-        self.0 .0
-    }
-}
 impl<F: Float> Wrap<F, Degrees> {
     /// Gets the value of this angle in degrees.
     #[inline]
@@ -485,100 +643,6 @@ impl<F: Float> Wrap<F, Degrees> {
     }
 }
 
-impl<F: Float, U: Unit<F>> From<Wrap<F, U>> for Angle<F, U> {
-    #[inline]
-    fn from(val: Wrap<F, U>) -> Self {
-        val.0
-    }
-}
-
-impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> Add<Rhs> for Wrap<F, U> {
-    type Output = Self;
-    #[inline]
-    fn add(self, rhs: Rhs) -> Self {
-        Self::wrap(self.val_raw() + rhs.into().0)
-    }
-}
-impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> AddAssign<Rhs> for Wrap<F, U> {
-    #[inline]
-    fn add_assign(&mut self, rhs: Rhs) {
-        *self = *self + rhs
-    }
-}
-impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> Sub<Rhs> for Wrap<F, U> {
-    type Output = Self;
-    #[inline]
-    fn sub(self, rhs: Rhs) -> Self {
-        Self::wrap(self.val_raw() + rhs.into().0)
-    }
-}
-impl<F: Float, U: Unit<F>, Rhs: Into<Angle<F, U>>> SubAssign<Rhs> for Wrap<F, U> {
-    #[inline]
-    fn sub_assign(&mut self, rhs: Rhs) {
-        *self = *self - rhs;
-    }
-}
-impl<F: Float, U: Unit<F>> Neg for Wrap<F, U> {
-    type Output = Self;
-    #[inline]
-    fn neg(self) -> Self {
-        Self::wrap(-self.val_raw())
-    }
-}
-
-impl<F: Float, U: Unit<F>> Mul<F> for Wrap<F, U> {
-    type Output = Self;
-    #[inline]
-    fn mul(self, rhs: F) -> Self {
-        Self::wrap(self.val_raw() * rhs)
-    }
-}
-impl<F: Float, U: Unit<F>> MulAssign<F> for Wrap<F, U> {
-    #[inline]
-    fn mul_assign(&mut self, rhs: F) {
-        *self = *self * rhs;
-    }
-}
-impl<F: Float, U: Unit<F>> Div<F> for Wrap<F, U> {
-    type Output = Self;
-    #[inline]
-    fn div(self, rhs: F) -> Self {
-        Self::wrap(self.val_raw() / rhs)
-    }
-}
-impl<F: Float, U: Unit<F>> DivAssign<F> for Wrap<F, U> {
-    #[inline]
-    fn div_assign(&mut self, rhs: F) {
-        *self = *self / rhs
-    }
-}
-
-impl<F: Float + fmt::Debug, U: Unit<F>> fmt::Debug for Wrap<F, U> {
-    #[inline]
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        fmt::Debug::fmt(&self.0, f)
-    }
-}
-impl<F: Float + fmt::Display, U: Unit<F>> fmt::Display for Wrap<F, U> {
-    #[inline]
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        fmt::Display::fmt(&self.0, f)
-    }
-}
-
-/// A 32-bit angle measured in radians.
-///
-/// This type is guaranteed to be finite (in debug mode). As such, it implements total equality and ordering.
-///
-/// When formatting with [`Display`](fmt::Display), it will be shown as a multiple of π.
-pub type Rad32 = Rad<f32>;
-/// A 64-bit angle measured in radians.
-///
-/// This type is guaranteed to be finite (in debug mode). As such, it implements total equality and ordering.
-///
-/// When formatting with [`Display`](fmt::Display), it will be shown as a multiple of π.
-pub type Rad64 = Rad<f64>;
-
 /// A 32-bit angle measured in degrees.
 ///
 /// This type is guaranteed to be finite (in debug mode). As such, it implements total equality and ordering.
@@ -616,7 +680,7 @@ mod tests {
         };
     }
 
-    use std::f64::consts::PI;
+    use std::f64::consts::{PI, SQRT_2};
 
     #[test]
     fn consts() {
@@ -752,6 +816,90 @@ mod tests {
         assert_epsilon!(val.val(), PI / 4.0);
     }
 
+    #[test]
+    fn trig() {
+        assert_epsilon!(Rad64::ZERO.sin(), 0.0);
+        assert_epsilon!(Rad64::ZERO.cos(), 1.0);
+        assert_epsilon!(Rad64::ZERO.tan(), 0.0);
+        assert_epsilon!(Wrap64::ZERO.sin(), 0.0);
+        assert_epsilon!(Wrap64::ZERO.cos(), 1.0);
+        assert_epsilon!(Wrap64::ZERO.tan(), 0.0);
+
+        assert_epsilon!((Rad64::HALF_TURN / 4.0).sin(), SQRT_2.recip());
+        assert_epsilon!((Rad64::HALF_TURN / 4.0).cos(), SQRT_2.recip());
+        assert_epsilon!((Rad64::HALF_TURN / 4.0).tan(), 1.0);
+        assert_epsilon!((Wrap64::HALF_TURN / 4.0).sin(), SQRT_2.recip());
+        assert_epsilon!((Wrap64::HALF_TURN / 4.0).cos(), SQRT_2.recip());
+        assert_epsilon!((Wrap64::HALF_TURN / 4.0).tan(), 1.0);
+
+        assert_epsilon!(Rad64::QUARTER_TURN.sin(), 1.0);
+        assert_epsilon!(Rad64::QUARTER_TURN.cos(), 0.0);
+        assert_epsilon!(Wrap64::QUARTER_TURN.sin(), 1.0);
+        assert_epsilon!(Wrap64::QUARTER_TURN.cos(), 0.0);
+
+        assert_epsilon!(Rad64::HALF_TURN.sin(), 0.0);
+        assert_epsilon!(Rad64::HALF_TURN.cos(), -1.0);
+        assert_epsilon!(Rad64::HALF_TURN.tan(), 0.0);
+        assert_epsilon!(Wrap64::HALF_TURN.sin(), 0.0);
+        assert_epsilon!(Wrap64::HALF_TURN.cos(), -1.0);
+        assert_epsilon!(Wrap64::HALF_TURN.tan(), 0.0);
+
+        assert_epsilon!((-Rad64::HALF_TURN / 4.0).sin(), -SQRT_2.recip());
+        assert_epsilon!((-Rad64::HALF_TURN / 4.0).cos(), SQRT_2.recip());
+        assert_epsilon!((-Rad64::HALF_TURN / 4.0).tan(), -1.0);
+        assert_epsilon!((Wrap64::HALF_TURN / -4.0).sin(), -SQRT_2.recip());
+        assert_epsilon!((Wrap64::HALF_TURN / -4.0).cos(), SQRT_2.recip());
+        assert_epsilon!((Wrap64::HALF_TURN / -4.0).tan(), -1.0);
+
+        assert_epsilon!((-Rad64::QUARTER_TURN).sin(), -1.0);
+        assert_epsilon!((-Rad64::QUARTER_TURN).cos(), 0.0);
+        assert_epsilon!((-Wrap64::QUARTER_TURN).sin(), -1.0);
+        assert_epsilon!((-Wrap64::QUARTER_TURN).cos(), 0.0);
+    }
+
+    #[test]
+    fn inverse_trig() {
+        assert_epsilon!(Rad64::asin(0.0).val(), 0.0);
+        assert_epsilon!(Rad64::acos(0.0).val(), PI / 2.0);
+        assert_epsilon!(Wrap64::asin(0.0).val(), 0.0);
+        assert_epsilon!(Wrap64::acos(0.0).val(), PI / 2.0);
+
+        assert_epsilon!(Rad64::asin(SQRT_2 / 2.0).val(), PI / 4.0);
+        assert_epsilon!(Rad64::acos(SQRT_2 / 2.0).val(), PI / 4.0);
+        assert_epsilon!(Wrap64::asin(SQRT_2 / 2.0).val(), PI / 4.0);
+        assert_epsilon!(Wrap64::acos(SQRT_2 / 2.0).val(), PI / 4.0);
+
+        assert_epsilon!(Rad64::asin(1.0).val(), PI / 2.0);
+        assert_epsilon!(Rad64::acos(1.0).val(), 0.0);
+        assert_epsilon!(Wrap64::asin(1.0).val(), PI / 2.0);
+        assert_epsilon!(Wrap64::acos(1.0).val(), 0.0);
+
+        assert_epsilon!(Rad64::asin(-SQRT_2 / 2.0).val(), -PI / 4.0);
+        assert_epsilon!(Rad64::acos(-SQRT_2 / 2.0).val(), 3.0 * PI / 4.0);
+        assert_epsilon!(Wrap64::asin(-SQRT_2 / 2.0).val(), -PI / 4.0);
+        assert_epsilon!(Wrap64::acos(-SQRT_2 / 2.0).val(), 3.0 * PI / 4.0);
+
+        assert_epsilon!(Rad64::asin(-1.0).val(), -PI / 2.0);
+        assert_epsilon!(Rad64::acos(-1.0).val(), PI);
+        assert_epsilon!(Wrap64::asin(-1.0).val(), -PI / 2.0);
+        assert_epsilon!(Wrap64::acos(-1.0).val(), PI);
+
+        assert_epsilon!(Rad64::atan2(0.0, 1.0).val(), 0.0);
+        assert_epsilon!(Rad64::atan2(SQRT_2 / 2.0, SQRT_2 / 2.0).val(), PI / 4.0);
+        assert_epsilon!(Rad64::atan2(1.0, 0.0).val(), PI / 2.0);
+        assert_epsilon!(
+            Rad64::atan2(SQRT_2 / 2.0, -SQRT_2 / 2.0).val(),
+            3.0 * PI / 4.0
+        );
+        assert_epsilon!(Rad64::atan2(0.0, -1.0).val(), PI);
+        assert_epsilon!(Rad64::atan2(-SQRT_2 / 2.0, SQRT_2 / 2.0).val(), -PI / 4.0);
+        assert_epsilon!(Rad64::atan2(-1.0, 0.0).val(), -PI / 2.0);
+        assert_epsilon!(
+            Rad64::atan2(-SQRT_2 / 2.0, -SQRT_2 / 2.0).val(),
+            -3.0 * PI / 4.0
+        );
+    }
+
     #[test]
     fn display() {
         assert_eq!(format!("{}", Rad64::ZERO), "0");
